/**
 * 基础拓展
 * @author yutent<yutent.io@gmail.com>
 * @date 2020/10/14 17:52:44
 */

import { offset } from 'wkit'

var placeholder = '在此输入文本'

function trim(str, sign) {
  return str.replace(new RegExp('^' + sign + '|' + sign + '$', 'g'), '')
}

function docScroll(k = 'X') {
  return window[`page${k.toUpperCase()}Offset`]
}

// 通用的弹层触发
function showDialog(dialog, elem) {
  var { left, top } = offset(elem)
  left -= docScroll('X')
  top += 29 - docScroll('Y')
  left += 'px'
  top += 'px'
  dialog.moveTo({ top, left })
  dialog.show()
  return Promise.resolve(dialog)
}

export default {
  header(elem) {
    this.$refs.header.classList.add('fadein')
  },

  h(level) {
    var wrap = this.selection(true) || placeholder
    wrap = wrap.replace(/^(#+ )?/, '#'.repeat(level) + ' ')
    this.insert(wrap, true)
  },

  quote(elem) {
    var wrap = this.selection(true) || placeholder
    wrap = wrap.replace(/^(>+ )?/, '> ')

    this.insert(wrap, true)
  },

  bold(elem) {
    var wrap = this.selection() || placeholder
    var unwrap = trim(wrap, '\\*\\*')
    wrap = wrap === unwrap ? `**${wrap}**` : unwrap
    this.insert(wrap, true)
  },

  italic(elem) {
    var wrap = this.selection() || placeholder
    var unwrap = trim(wrap, '_')
    wrap = wrap === unwrap ? `_${wrap}_` : unwrap
    this.insert(wrap, true)
  },

  through(elem) {
    var wrap = this.selection() || placeholder
    var unwrap = trim(wrap, '~~')
    wrap = wrap === unwrap ? `~~${wrap}~~` : unwrap
    this.insert(wrap, true)
  },

  list(elem) {
    var wrap = this.selection(true) || placeholder

    wrap = wrap.replace(/^([+\-*] )?/, '+ ')
    this.insert(wrap, true)
  },

  order(elem) {
    var wrap = this.selection(true) || placeholder

    wrap = wrap.replace(/^(\d+\. )?/, '1. ')
    this.insert(wrap, true)
  },

  line(elem) {
    this.insert('\n\n---\n\n', false)
  },

  code(elem) {
    var wrap = this.selection() || placeholder
    var unwrap = trim(wrap, '`')
    wrap = wrap === unwrap ? `\`${wrap}\`` : unwrap
    this.insert(wrap, true)
  },

  codeblock(elem) {
    this.insert('\n```language\n\n```\n')
  },

  table(elem) {
    // showDialog(this.__TABLE_ADDON__, elem)
    this.$refs.table.classList.add('fadein')
  },

  link(elem) {
    showDialog(this.__LINK_ADDON__, elem).then(dialog => {
      var wrap = this.selection() || placeholder
      dialog.__txt__.value = wrap
    })
  },

  image(elem) {
    var $file = this.__ATTACH_ADDON__.querySelector('input')

    this._attach = 'image'
    $file.setAttribute('accept', 'image/*')

    showDialog(this.__ATTACH_ADDON__, elem)
  },

  attach(elem) {
    var $file = this.__ATTACH_ADDON__.querySelector('input')
    this._attach = 'file'
    $file.removeAttribute('accept')
    showDialog(this.__ATTACH_ADDON__, elem)
  },

  fullscreen(elem) {
    this.classList.toggle('fullscreen')
    elem.classList.toggle('active')
  },
  preview(elem) {
    this.previewEnabled = !this.previewEnabled
    this.$refs.view.classList.toggle('active')
    elem.classList.toggle('active')
    if (this.previewEnabled) {
      this.$refs.view.code = this.value
    }
  }
}
